@@ -1,4 +1,5 @@
 from time import time, sleep
+from enum import Enum
 import logging
 from queue import Queue
 from pymodbus.client.sync import ModbusTcpClient, ModbusSocketFramer
@@ -13,9 +14,13 @@ DEFAULT_WRITE_BLOCK_INTERVAL_S = 0.2
 DEFAULT_WRITE_SLEEP_S = 0.05
 DEFAULT_READ_SLEEP_S = 0.05
 
+class WordOrder(Enum):
+    HighLow = 1
+    LowHigh = 2
+
 class modbus_interface():
 
-    def __init__(self, ip, port=502, update_rate_s=DEFAULT_SCAN_RATE_S, variant=None, scan_batching=None):
+    def __init__(self, ip, port=502, update_rate_s=DEFAULT_SCAN_RATE_S, variant=None, scan_batching=None, word_order=WordOrder.HighLow):
         self._ip = ip
         self._port = port
         # This is a dict of sets. Each key represents one table of modbus registers.
@@ -29,6 +34,7 @@ class modbus_interface():
         self._writing = False
         self._variant = variant
         self._scan_batching = DEFAULT_SCAN_BATCHING
+        self._word_order = word_order
         if scan_batching is not None:
             if scan_batching < MIN_SCAN_BATCHING:
                 logging.warning("Bad value for scan_batching: {}. Enforcing minimum value of {}".format(scan_batching, MIN_SCAN_BATCHING))
@@ -53,11 +59,14 @@ class modbus_interface():
                                        framer=ModbusSocketFramer, timeout=1,
                                        RetryOnEmpty=True, retries=1)
 
-    def add_monitor_register(self, table, addr):
+    def add_monitor_register(self, table, addr, type='uint16'):
         # Accepts a modbus register and table to monitor
         if table not in self._tables:
             raise ValueError("Unsupported table type. Please only use: {}".format(self._tables.keys()))
-        self._tables[table].add(addr)
+        # Register enough sequential addresses to fill the size of the register type.
+        # Note: Each address provides 2 bytes of data.
+        for i in range(type_length(type)):
+            self._tables[table].add(addr+i)
 
     def poll(self):
         # Polls for the values marked as interesting in self._tables.
@@ -79,19 +88,41 @@ class modbus_interface():
                     start = group + self._scan_batching-1
         self._process_writes()
 
-    def get_value(self, table, addr):
+    def get_value(self, table, addr, type='uint16'):
         if table not in self._values:
             raise ValueError("Unsupported table type. Please only use: {}".format(self._values.keys()))
         if addr not in self._values[table]:
             raise ValueError("Unpolled address. Use add_monitor_register(addr, table) to add a register to the polled list.")
-        return self._values[table][addr]
+        # Read sequential addresses to get enough bytes to satisfy the type of this register.
+        # Note: Each address provides 2 bytes of data.
+        value = bytes(0)
+        type_len = type_length(type)
+        for i in range(type_len):
+            if self._word_order == WordOrder.HighLow:
+                data = self._values[table][addr + i]
+            else:
+                data = self._values[table][addr + (type_len-i-1)]
+            value += data.to_bytes(2,'big')
+        value = _convert_from_bytes_to_type(value, type)
+        return value
 
-    def set_value(self, table, addr, value, mask=0xFFFF):
+    def set_value(self, table, addr, value, mask=0xFFFF, type='uint16'):
         if table != 'holding':
             # I'm not sure if this is true for all devices. I might support writing to coils later,
             # so leave this door open.
             raise ValueError("Can only set values in the holding table.")
-        self._planned_writes.put((addr, value, mask))
+
+        bytes_to_write = _convert_from_type_to_bytes(value, type)
+        # Put the bytes into _planned_writes stitched into two-byte pairs
+
+        type_len = type_length(type)
+        for i in range(type_len):
+            if self._word_order == WordOrder.HighLow:
+                value = _convert_from_bytes_to_type(bytes_to_write[i*2:i*2+2], 'uint16')
+            else:
+                value = _convert_from_bytes_to_type(bytes_to_write[(type_len-i-1)*2:(type_len-i-1)*2+2], 'uint16')
+            self._planned_writes.put((addr+i, value, mask))
+
         self._process_writes()
 
     def _process_writes(self, max_block_s=DEFAULT_WRITE_BLOCK_INTERVAL_S):
@@ -144,22 +175,33 @@ class modbus_interface():
             # The result doesn't have a registers attribute, something has gone wrong!
             raise ValueError("Failed to read {} {} table registers starting from {}: {}".format(count, table, start, result))
 
-def _convert_from_uint16_to_type(value, type):
-    type = type.strip().lower()
-    if type == 'uint16':
-        return value
-    elif type == 'int16':
-        if value >= 2**15:
-            return value - 2**16
-        return value
-    raise ValueError("Unrecognised type conversion attempted: uint16 to {}".format(type))
+def type_length(type):
+    # Return the number of addresses needed for the type.
+    # Note: Each address provides 2 bytes of data.
+    if type in ['int16', 'uint16']:
+        return 1
+    elif type in ['int32', 'uint32']:
+        return 2
+    elif type in ['int64', 'uint64']:
+        return 4
+    raise ValueError ("Unsupported type {}".format(type))
 
-def _convert_from_type_to_uint16(value, type):
+def type_signed(type):
+    # Returns whether the provided type is signed
+    if type in ['uint16', 'uint32', 'uint64']:
+        return False
+    elif type in ['int16', 'int32', 'int64']:
+        return True
+    raise ValueError ("Unsupported type {}".format(type))
+
+def _convert_from_bytes_to_type(value, type):
     type = type.strip().lower()
-    if type == 'uint16':
-        return value
-    elif type == 'int16':
-        if value < 0:
-            return value + 2**16
-        return value
-    raise ValueError("Unrecognised type conversion attempted: {} to uint16".format(type))
\ No newline at end of file
+    signed = type_signed(type)
+    return int.from_bytes(value,byteorder='big',signed=signed)
+
+def _convert_from_type_to_bytes(value, type):
+    type = type.strip().lower()
+    signed = type_signed(type)
+    # This can throw an OverflowError in various conditons. This will usually
+    # percolate upwards and spit out an exception from on_message.
+    return int(value).to_bytes(type_length(type)*2,byteorder='big',signed=signed)
